@@ -129,7 +129,7 @@ class Connection(_Connection, DBusProperties):
                 if not isinstance(value, (int, long)):
                     raise InvalidArgument('incorrect type to %s parameter, got %s, expected an int' % (parm, type(value)))
             elif sig == 'b':
-                if not isinstance(value, bool):
+                if not isinstance(value, (bool, dbus.Boolean)):
                     raise InvalidArgument('incorrect type to %s parameter, got %s, expected an boolean' % (parm, type(value)))
             else:
                 raise TypeError('unknown type signature %s in protocol parameters' % type)
@@ -178,9 +178,12 @@ class Connection(_Connection, DBusProperties):
     def set_self_handle(self, handle):
         self._self_handle = handle
 
-    def get_channel_path(self):
-        ret = '%s/channel%d' % (self._object_path, self._next_channel_id)
-        self._next_channel_id += 1
+    def get_channel_path(self, suffix):
+        if not suffix:
+            ret = '%s/channel%d' % (self._object_path, self._next_channel_id)
+            self._next_channel_id += 1
+        else:
+            ret = '%s/%s' % (self._object_path, suffix)
         return ret
 
     def add_channels(self, channels, signal=True):
@@ -309,7 +312,7 @@ class Connection(_Connection, DBusProperties):
         self.check_connected()
         ret = []
         for channel in self._channels:
-            chan = (channel._object_path, channel._type, channel._get_handle_type(), channel._handle)
+            chan = (channel._object_path, channel._type, channel._handle.get_type(), channel._handle.get_id())
             ret.append(chan)
         return ret
 
@@ -416,7 +419,7 @@ class ConnectionInterfaceRequests(
 
         # Allow TargetHandleType to be missing, but not to be otherwise broken.
         check_valid_type_if_exists('TargetHandleType',
-            lambda p: p > 0 and p < (2**32)-1)
+            lambda p: p >= 0 and p <= LAST_HANDLE_TYPE)
 
         # Allow TargetType to be missing, but not to be otherwise broken.
         check_valid_type_if_exists('TargetHandle',
@@ -483,8 +486,8 @@ class ConnectionInterfaceRequests(
                     if handle.get_name() == target_id and handle.get_type() == target_handle_type:
                         target_handle = handle.get_id()
                 if not target_handle:
-                    raise InvalidHandle('TargetID %s not valid for type %d' %
-                        target_id, target_handle_type)
+                    raise InvalidHandle('TargetID %s not valid for type %d' % (
+                        target_id, target_handle_type))
 
                 altered_properties[CHANNEL_INTERFACE + '.TargetHandle'] = \
                     target_handle
@@ -511,20 +514,8 @@ class ConnectionInterfaceRequests(
 
         channel = self._channel_manager.create_channel_for_props(props, signal=False)
 
-        # Remove mutable properties
-        todel = []
-        for prop in props:
-            iface, name = prop.rsplit('.', 1) # a bit of a hack
-            if name in channel._immutable_properties:
-                if channel._immutable_properties[name] != iface:
-                    todel.append(prop)
-            else:
-                todel.append(prop)
-
-        for p in todel:
-            del props[p]
-
-        _success(channel._object_path, props)
+        returnedProps = channel.get_props()
+        _success(channel._object_path, returnedProps)
 
         # CreateChannel MUST return *before* NewChannels is emitted.
         self.signal_new_channels([channel])
@@ -541,7 +532,8 @@ class ConnectionInterfaceRequests(
 
         channel = self._channel_manager.channel_for_props(props, signal=False)
 
-        _success(yours, channel._object_path, props)
+        returnedProps = channel.get_props()
+        _success(yours, channel._object_path, returnedProps)
 
         self.signal_new_channels([channel])
 
